@@ -18,8 +18,6 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *)
 
-(* $Id$ *)
-
 type protocol =
   | SSLv23
   | SSLv3
@@ -157,6 +155,8 @@ external create_context : protocol -> context_type -> context = "ocaml_ssl_creat
 
 external use_certificate : context -> string -> string -> unit = "ocaml_ssl_ctx_use_certificate"
 
+external use_certificate_from_string : context -> string -> string -> unit = "ocaml_ssl_ctx_use_certificate_from_string"
+
 external set_password_callback : context -> (bool -> string) -> unit = "ocaml_ssl_ctx_set_default_passwd_cb"
 
 external embed_socket : Unix.file_descr -> context -> socket = "ocaml_ssl_embed_socket"
@@ -177,6 +177,8 @@ external set_default_verify_paths : context -> bool = "ocaml_ssl_ctx_set_default
 
 external get_verify_result : socket -> int = "ocaml_ssl_get_verify_result"
 
+external get_verify_error_string : int -> string = "ocaml_ssl_get_verify_error_string"
+
 type verify_mode =
   | Verify_peer
   | Verify_fail_if_no_peer_cert
@@ -200,6 +202,8 @@ external set_context_alpn_protos : context -> string list -> unit = "ocaml_ssl_c
 
 external set_context_alpn_select_callback : context -> (string list -> string option) -> unit = "ocaml_ssl_ctx_set_alpn_select_callback"
 
+external version : socket -> protocol = "ocaml_ssl_version"
+
 type cipher
 
 external get_cipher : socket -> cipher = "ocaml_ssl_get_current_cipher"
@@ -222,6 +226,10 @@ external get_issuer : certificate -> string = "ocaml_ssl_get_issuer"
 
 external get_subject : certificate -> string = "ocaml_ssl_get_subject"
 
+external get_start_date : certificate -> Unix.tm = "ocaml_ssl_get_start_date"
+
+external get_expiration_date : certificate -> Unix.tm = "ocaml_ssl_get_expiration_date"
+
 external file_descr_of_socket : socket -> Unix.file_descr = "ocaml_ssl_get_file_descr"
 
 external set_client_SNI_hostname : socket -> string -> unit = "ocaml_ssl_set_client_SNI_hostname"
