@@ -24,8 +24,6 @@
     @author Samuel Mimram
 *)
 
-(* $Id$ *)
-
 (** {1 Exceptions and errors} *)
 
 (** An ssl error has occured (see SSL_get_error(3ssl) for details). *)
@@ -245,6 +243,11 @@ val create_context : protocol -> context_type -> context
   * name. *)
 val use_certificate : context -> string -> string -> unit
 
+(** Use a certificate whose contents is given as argument (you should use
+    instead [use_certificate] if you want to read the certificate from a
+    file). *)
+val use_certificate_from_string : context -> string -> string -> unit
+
 (** Set the callback function called to get passwords for encrypted PEM files.
   * The callback function takes a boolean argument which indicates if it's used
   * for reading/decryption ([false]) or writing/encryption ([true]).
@@ -273,13 +276,15 @@ val set_client_verify_callback_verbose : bool -> unit
   * Warning: this might change in the future. *)
 val set_verify : context -> verify_mode list -> verify_callback option -> unit
 
-(** Set the maximum depth for the certificate chain verification that shall be allowed. *)
+(** Set the maximum depth for the certificate chain verification that shall be
+    allowed. *)
 val set_verify_depth : context -> int -> unit
 
 (** Set the list of supported ALPN protocols for negotiation to the context. *)
 val set_context_alpn_protos : context -> string list -> unit
 
-(** Set the callback to allow server to select the preferred protocol from client's available protocols. *)
+(** Set the callback to allow server to select the preferred protocol from
+    client's available protocols. *)
 val set_context_alpn_select_callback : context -> (string list -> string option) -> unit
 
 
@@ -295,7 +300,8 @@ type cipher
   * *)
 val disable_protocols : context -> protocol list -> unit
 
-(** Set the list of available ciphers for a context. See man ciphers(1) for the format of the string. *)
+(** Set the list of available ciphers for a context. See man ciphers(1) for the
+    format of the string. *)
 val set_cipher_list : context -> string -> unit
 
 (** When choosing a cipher, use the server's preferences instead of the client
@@ -323,6 +329,14 @@ val get_cipher_name : cipher -> string
 (** Get the version of a cipher. *)
 val get_cipher_version : cipher -> string
 
+(** Get the version used for the connection. As per the
+   {{:https://www.openssl.org/docs/man1.1.1/man3/SSL_get_version.html} OpenSSL
+   documentation}, should only be called after the initial handshake has been
+   completed. Prior to that the results returned from these functions may be
+   unreliable.
+
+  @raise Failure if the version is unknown *)
+val version : socket -> protocol
 
 (** {2 Certificates} *)
 
@@ -343,6 +357,12 @@ val get_issuer : certificate -> string
 (** Get the subject of a certificate. *)
 val get_subject : certificate -> string
 
+(** Get the start date of a certificate. *)
+val get_start_date : certificate -> Unix.tm
+
+(** Get the expiration date of a certificate. *)
+val get_expiration_date : certificate -> Unix.tm
+
 (** [load_verify_locations ctxt cafile capath] specifies the locations for the
     context [ctx], at which CA certificates for verification purposes are
     located. [cafile] should be the name of a CA certificates file in PEM format
@@ -361,6 +381,9 @@ val set_default_verify_paths : context -> bool
 (** Get the verification result. *)
 val get_verify_result : socket -> int
 
+(** Get a human readable verification error message for the verification error
+    Its input should be the result of calling [get_verify_result]. *)
+val get_verify_error_string : int -> string
 
 (** {2 Creating, connecting and closing sockets} *)
 
@@ -443,14 +466,12 @@ val write : socket -> Bytes.t -> int -> int -> int
 val write_substring : socket -> string -> int -> int -> int
 
 (** [write_bigarray sock ba off len] sends data over a connected SSL socket.
-    This function releases the runtime while the read takes place.
-  *)
+    This function releases the runtime while the read takes place. *)
 val write_bigarray : socket -> bigarray -> int -> int -> int
 
 (** [write_bigarray sock ba off len] sends data over a connected SSL socket.
     This function DOES NOT release the runtime while the read takes place: it
-    must be used with nonblocking sockets.
-  *)
+    must be used with nonblocking sockets. *)
 val write_bigarray_blocking : socket -> bigarray -> int -> int -> int
 
 
