@@ -470,6 +470,42 @@ CAMLprim value ocaml_ssl_ctx_use_certificate(value context, value cert, value pr
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_ssl_ctx_use_certificate_from_string(value context, value cert, value privkey)
+{
+  CAMLparam3(context, cert, privkey);
+  SSL_CTX *ctx = Ctx_val(context);
+  char *cert_data = String_val(cert);
+  int cert_data_length = caml_string_length(cert);
+  char *privkey_data = String_val(privkey);
+  int privkey_data_length = caml_string_length(privkey);
+  char buf[256];
+  X509 *x509_cert = NULL;
+  RSA *rsa = NULL;
+  BIO *cbio, *kbio;
+
+  cbio = BIO_new_mem_buf((void*)cert_data, cert_data_length);
+  x509_cert = PEM_read_bio_X509(cbio, NULL, 0, NULL);
+  if (NULL == x509_cert || SSL_CTX_use_certificate(ctx, x509_cert) <= 0)
+  {
+    ERR_error_string_n(ERR_get_error(), buf, sizeof(buf));
+    caml_raise_with_arg(*caml_named_value("ssl_exn_certificate_error"), caml_copy_string(buf));
+  }
+
+  kbio = BIO_new_mem_buf((void*)privkey_data, privkey_data_length);
+  rsa = PEM_read_bio_RSAPrivateKey(kbio, NULL, 0, NULL);
+  if (NULL == rsa || SSL_CTX_use_RSAPrivateKey(ctx, rsa) <= 0)
+  {
+    ERR_error_string_n(ERR_get_error(), buf, sizeof(buf));
+    caml_raise_with_arg(*caml_named_value("ssl_exn_private_key_error"), caml_copy_string(buf));
+  }
+  if (!SSL_CTX_check_private_key(ctx))
+  {
+    caml_raise_constant(*caml_named_value("ssl_exn_unmatching_keys"));
+  }
+
+  CAMLreturn(Val_unit);
+}
+
 CAMLprim value ocaml_ssl_get_verify_result(value socket)
 {
   CAMLparam1(socket);
@@ -483,6 +519,18 @@ CAMLprim value ocaml_ssl_get_verify_result(value socket)
   CAMLreturn(Val_int(ans));
 }
 
+CAMLprim value ocaml_ssl_get_verify_error_string(value verrn)
+{
+  int errn = Int_val(verrn);
+  const char *error_string;
+
+  caml_enter_blocking_section();
+  error_string = X509_verify_cert_error_string(errn);
+  caml_leave_blocking_section();
+
+  return caml_copy_string(error_string);
+}
+
 CAMLprim value ocaml_ssl_get_client_verify_callback_ptr(value unit)
 {
   return (value)client_verify_callback;
@@ -674,8 +722,11 @@ static int alpn_select_cb(SSL *ssl,
   protocol_list = build_alpn_protocol_list(in, inlen);
   selected_protocol_opt = caml_callback(*((value*)arg), protocol_list);
 
-  if (selected_protocol_opt == Val_none)
+  if(selected_protocol_opt == Val_none)
+  {
+    caml_enter_blocking_section();
     return SSL_TLSEXT_ERR_NOACK;
+  }
 
   selected_protocol = Field(selected_protocol_opt, 0);
   len = caml_string_length(selected_protocol);
@@ -798,6 +849,45 @@ CAMLprim value ocaml_ssl_disable_protocols(value context, value protocol_list)
   CAMLreturn(Val_unit);
 }
 
+CAMLprim value ocaml_ssl_version(value socket)
+{
+  CAMLparam1(socket);
+  SSL *ssl = SSL_val(socket);
+  int version;
+  int ret;
+
+  caml_enter_blocking_section();
+  version = SSL_version(ssl);
+  caml_leave_blocking_section();
+
+  switch(version) {
+    case SSL3_VERSION:
+      ret = 1;
+      break;
+
+    case TLS1_VERSION:
+      ret = 2;
+      break;
+
+    case TLS1_1_VERSION:
+      ret = 3;
+      break;
+
+    case TLS1_2_VERSION:
+      ret = 4;
+      break;
+
+    case TLS1_3_VERSION:
+      ret = 5;
+      break;
+
+    default:
+      caml_failwith("Ssl.version");
+  }
+
+  CAMLreturn(Val_int(ret));
+}
+
 CAMLprim value ocaml_ssl_get_current_cipher(value socket)
 {
   CAMLparam1(socket);
@@ -1039,6 +1129,48 @@ CAMLprim value ocaml_ssl_get_subject(value certificate)
   CAMLreturn(caml_copy_string(subject));
 }
 
+static value alloc_tm(struct tm *tm)
+{
+  value res;
+  res = caml_alloc_small(9, 0);
+  Field(res,0) = Val_int(tm->tm_sec);
+  Field(res,1) = Val_int(tm->tm_min);
+  Field(res,2) = Val_int(tm->tm_hour);
+  Field(res,3) = Val_int(tm->tm_mday);
+  Field(res,4) = Val_int(tm->tm_mon);
+  Field(res,5) = Val_int(tm->tm_year);
+  Field(res,6) = Val_int(tm->tm_wday);
+  Field(res,7) = Val_int(tm->tm_yday);
+  Field(res,8) = tm->tm_isdst ? Val_true : Val_false;
+  return res;
+}
+
+CAMLprim value ocaml_ssl_get_start_date(value certificate)
+{
+  CAMLparam1(certificate);
+  X509 *cert = Cert_val(certificate);
+  struct tm t;
+
+  caml_enter_blocking_section();
+  ASN1_TIME_to_tm(X509_get0_notBefore(cert), &t);
+  caml_leave_blocking_section();
+
+  CAMLreturn(alloc_tm(&t));
+}
+
+CAMLprim value ocaml_ssl_get_expiration_date(value certificate)
+{
+  CAMLparam1(certificate);
+  X509 *cert = Cert_val(certificate);
+  struct tm t;
+
+  caml_enter_blocking_section();
+  ASN1_TIME_to_tm(X509_get0_notAfter(cert), &t);
+  caml_leave_blocking_section();
+
+  CAMLreturn(alloc_tm(&t));
+}
+
 CAMLprim value ocaml_ssl_ctx_load_verify_locations(value context, value ca_file, value ca_path)
 {
   CAMLparam3(context, ca_file, ca_path);
